@@ -525,6 +525,33 @@ ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
     if(fbuf_init(&f)){
       return -1;
     }
+    if(cursor_yx_get(n->tcache.ttyfd, u7, &y, NULL)){
+      return -1;
+    }
+    if(toty - dimy < y){
+      int scrolls = y - 1;
+      y = toty - dimy;
+      if(y < 0){
+        y = 0;
+      }
+      scrolls -= y;
+      // perform our scrolling outside of the fbuf framework, as we need it
+      // to happen immediately for fbdcon
+      if(ncdirect_cursor_move_yx(n, y, xoff)){
+        return -1;
+      }
+      const char* indn = get_escape(&n->tcache, ESCAPE_IND);
+      if(scrolls > 1 && indn){
+        if(term_emit(tiparm(indn, scrolls), stdout, true) < 0){
+          return -1;
+        }
+      }
+      while(scrolls--){
+        if(ncfputc('\v', stdout) < 0){
+          return -1;
+        }
+      }
+    }
     if(sprite_draw(&n->tcache, NULL, np->sprite, &f, y, xoff) < 0){
       return -1;
     }
@@ -540,6 +567,13 @@ ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
         return -1;
       }
     }
+    int targy = y + dimy;
+    if(targy > toty){
+      targy = toty;
+    }
+    if(ncdirect_cursor_move_yx(n, targy, xoff)){
+      return -1;
+    }
     return 0;
   }
 //fprintf(stderr, "rasterizing %dx%d+%d\n", dimy, dimx, xoff);
@@ -827,8 +861,12 @@ ncdirect_stop_minimal(void* vnc){
       ret |= nc->tcache.pixel_shutdown(&f);
     }
     ret |= reset_term_attributes(&nc->tcache, &f);
+    ret |= fbuf_finalize(&f, stdout);
   }
   if(nc->tcache.ttyfd >= 0){
+    if(nc->tcache.kittykbd){
+      ret |= tty_emit("\x1b[<u", nc->tcache.ttyfd);
+    }
     const char* cnorm = get_escape(&nc->tcache, ESCAPE_CNORM);
     if(cnorm && tty_emit(cnorm, nc->tcache.ttyfd)){
       ret = -1;
