@@ -183,7 +183,7 @@ static int
 ncinputlayer_add_input_escape(ncinputlayer* nc, const char* esc, uint32_t special,
                               unsigned shift, unsigned ctrl, unsigned alt){
   if(esc[0] != NCKEY_ESC || strlen(esc) < 2){ // assume ESC prefix + content
-    logerror("not an escape: %s (0x%x)\n", esc, special);
+    logerror("not an escape (0x%x)\n", special);
     return -1;
   }
   esctrie** cur = &nc->inputescapes;
@@ -213,7 +213,9 @@ ncinputlayer_add_input_escape(ncinputlayer* nc, const char* esc, uint32_t specia
   // it appears that multiple keys can be mapped to the same escape string. as
   // an example, see "kend" and "kc1" in st ("simple term" from suckless) :/.
   if((*cur)->special != NCKEY_INVALID){ // already had one here!
-    logwarn("already added escape (got 0x%x, wanted 0x%x)\n", (*cur)->special, special);
+    if((*cur)->special != special){
+      logwarn("already added escape (got 0x%x, wanted 0x%x)\n", (*cur)->special, special);
+    }
   }else{
     (*cur)->special = special;
     (*cur)->shift = shift;
@@ -243,6 +245,26 @@ handle_csi(ncinputlayer* nc, ncinput* ni, int leftmargin, int topmargin){
   while(nc->inputbuf_occupied){
     int candidate = pop_input_keypress(nc);
     logdebug("candidate: %c (%d)\n", candidate, candidate);
+    if(candidate == 'u'){ // kitty keyboard protocol
+      if(state == PARAM3){
+        logwarn("triparam kitty message?\n");
+        break;
+      }else if(state == PARAM1){
+        param1 = param;
+        param = 1;
+      }
+      ni->shift = !!((param - 1) & 0x1);
+      ni->alt = !!((param - 1) & 0x2);
+      ni->ctrl = !!((param - 1) & 0x4);
+      // FIXME decode remaining modifiers through 128
+      // standard keyboard protocol reports ctrl+ascii as the capital form,
+      // so (for now) conform with kitty protocol...
+      if(param1 < 128 && islower(param1) && ni->ctrl){
+        param1 = toupper(param1);
+      }
+      ni->id = param1;
+      return param1;
+    }
     if(state == PARAM1){
       // if !mouse and candidate is '>', set mouse. otherwise it ought be a
       // digit or a semicolon.
@@ -366,6 +388,7 @@ handle_getc(ncinputlayer* nc, int kpress, ncinput* ni, int leftmargin, int topma
   if(kpress < 0){
     return -1;
   }
+  unsigned csiidx = 0;
   if(kpress == NCKEY_ESC){
     const esctrie* esc = nc->inputescapes;
     int candidate = 0;
@@ -380,6 +403,9 @@ handle_getc(ncinputlayer* nc, int kpress, ncinput* ni, int leftmargin, int topma
       }
       candidate = pop_input_keypress(nc);
       logdebug("trie candidate: %c %d (%d)\n", candidate, esc->special, candidate);
+      if(csi){
+        nc->csibuf[csiidx++] = candidate;
+      }
       if(esc->trie == NULL){
         esc = NULL;
       }else if(candidate >= 0x80 || candidate < 0){
@@ -396,8 +422,9 @@ handle_getc(ncinputlayer* nc, int kpress, ncinput* ni, int leftmargin, int topma
       return esc->special;
     }
     if(csi){
-      // FIXME might need to unpop more than one
-      unpop_keypress(nc, candidate);
+      while(csiidx){
+        unpop_keypress(nc, nc->csibuf[--csiidx]);
+      }
       return handle_csi(nc, ni, leftmargin, topmargin);
     }
     // interpret it as alt + candidate FIXME broken for first char matching
@@ -444,17 +471,19 @@ handle_getc(ncinputlayer* nc, int kpress, ncinput* ni, int leftmargin, int topma
 static int
 block_on_input(tinfo* ti, const struct timespec* ts){
 #ifdef __MINGW64__
-  /* FIXME this approach doesn't work in MinTTY =\ */
-  int timeoutms = ts ? ts->tv_sec * 1000 + ts->tv_nsec / 1000000 : -1;
-  DWORD d = WaitForMultipleObjects(1, &ti->inhandle, FALSE, timeoutms);
-  if(d == WAIT_TIMEOUT){
-    return 0;
-  }else if(d == WAIT_FAILED){
+  if(ti->qterm == TERMINAL_MSTERMINAL){
+    int timeoutms = ts ? ts->tv_sec * 1000 + ts->tv_nsec / 1000000 : -1;
+    DWORD d = WaitForMultipleObjects(1, &ti->inhandle, FALSE, timeoutms);
+    if(d == WAIT_TIMEOUT){
+      return 0;
+    }else if(d == WAIT_FAILED){
+      return -1;
+    }else if(d - WAIT_OBJECT_0 == 0){
+      return 1;
+    }
     return -1;
-  }else if(d - WAIT_OBJECT_0 == 0){
-    return 1;
   }
-  return -1;
+  return 1; // ? FIXME
 #else
   struct pollfd pfd = {
     .fd = ti->input.infd,
@@ -465,7 +494,7 @@ block_on_input(tinfo* ti, const struct timespec* ts){
   pfd.events |= POLLRDHUP;
 #endif
   int events;
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__MINGW64__)
   int timeoutms = ts ? ts->tv_sec * 1000 + ts->tv_nsec / 1000000 : -1;
   while((events = poll(&pfd, 1, timeoutms)) < 0){ // FIXME smask?
 #else // linux, BSDs
@@ -475,7 +504,7 @@ block_on_input(tinfo* ti, const struct timespec* ts){
   sigdelset(&smask, SIGWINCH);
   while((events = ppoll(&pfd, 1, ts, &smask)) < 0){
 #endif
-    if(errno != EINTR && errno != EAGAIN){
+    if(errno != EINTR && errno != EAGAIN && errno != EBUSY && errno != EWOULDBLOCK){
       return -1;
     }
     if(resize_seen){
@@ -529,11 +558,50 @@ handle_queued_input(ncinputlayer* nc, ncinput* ni,
     }
     if(ret == NCKEY_CURSOR_LOCATION_REPORT){
       enqueue_cursor_report(nc, ni);
+    }else if(ni->ctrl && !ni->shift && !ni->alt){
+#ifndef __MINGW64__
+      if(ret == 'c'){
+        raise(SIGINT); // FIXME only if linesigs aren't disabled
+        continue;
+      }else if(ret == 'z'){
+        raise(SIGTSTP); // FIXME only if linesigs aren't disabled
+        continue;
+      }else if(ret == '\\'){
+        raise(SIGQUIT); // FIXME only if linesigs aren't disabled
+        continue;
+      }
+#endif
     }
   }while(ret == NCKEY_CURSOR_LOCATION_REPORT);
   return ret;
 }
 
+int ncinput_shovel(ncinputlayer* ni, const char* buf, size_t len){
+  int ret = -1;
+  pthread_mutex_lock(&ni->lock);
+  size_t space = input_queue_space(ni);
+  if(len < space){
+    size_t spaceback = sizeof(ni->inputbuf) / sizeof(*ni->inputbuf) - ni->inputbuf_write_at;
+    if(spaceback > len){
+      spaceback = len;
+    }
+    memcpy(ni->inputbuf + ni->inputbuf_write_at, buf, spaceback);
+    len -= spaceback;
+    ni->inputbuf_write_at += spaceback;
+    if(len){
+      memcpy(ni->inputbuf, buf + spaceback, len);
+      ni->inputbuf_write_at = len;
+    }
+    ni->inputbuf_occupied += len + spaceback;
+    ret = 0;
+  }
+  pthread_mutex_unlock(&ni->lock);
+  if(ret < 0){
+    logwarn("dropped %lluB event\n", (long long unsigned)len);
+  }
+  return ret;
+}
+
 // this is the only function which actually reads, and it can be called from
 // either our context (looking for cursor reports) or the user's. all it does
 // is attempt to fill up the input ringbuffer, exiting either when that
@@ -663,6 +731,28 @@ uint32_t ncdirect_getc(ncdirect* nc, const struct timespec *ts,
   return ncdirect_get(nc, ts, ni);
 }
 
+// https://sw.kovidgoyal.net/kitty/keyboard-protocol/#functional-key-definitions
+static int
+prep_kitty_special_keys(ncinputlayer* nc){
+  static const struct {
+    const char* esc;
+    uint32_t key;
+    bool shift, ctrl, alt;
+  } keys[] = {
+    { .esc = "\x1b[P", .key = NCKEY_F01, },
+    { .esc = "\x1b[Q", .key = NCKEY_F02, },
+    { .esc = "\x1b[R", .key = NCKEY_F03, },
+    { .esc = "\x1b[S", .key = NCKEY_F04, },
+    { .esc = NULL, .key = NCKEY_INVALID, },
+  }, *k;
+  for(k = keys ; k->esc ; ++k){
+    if(ncinputlayer_add_input_escape(nc, k->esc, k->key, k->shift, k->ctrl, k->alt)){
+      return -1;
+    }
+  }
+  return 0;
+}
+
 // add the hardcoded windows input sequences to ti->input. should only
 // be called after verifying that this is TERMINAL_MSTERMINAL.
 static int
@@ -675,32 +765,32 @@ prep_windows_special_keys(ncinputlayer* nc){
     uint32_t key;
     bool shift, ctrl, alt;
   } keys[] = {
-    { .esc = "\e[A", .key = NCKEY_UP, },
-    { .esc = "\e[B", .key = NCKEY_DOWN, },
-    { .esc = "\e[C", .key = NCKEY_RIGHT, },
-    { .esc = "\e[D", .key = NCKEY_LEFT, },
-    { .esc = "\e[1;5A", .key = NCKEY_UP, .ctrl = 1, },
-    { .esc = "\e[1;5B", .key = NCKEY_DOWN, .ctrl = 1, },
-    { .esc = "\e[1;5C", .key = NCKEY_RIGHT, .ctrl = 1, },
-    { .esc = "\e[1;5D", .key = NCKEY_LEFT, .ctrl = 1, },
-    { .esc = "\e[H", .key = NCKEY_HOME, },
-    { .esc = "\e[F", .key = NCKEY_END, },
-    { .esc = "\e[2~", .key = NCKEY_INS, },
-    { .esc = "\e[3~", .key = NCKEY_DEL, },
-    { .esc = "\e[5~", .key = NCKEY_PGUP, },
-    { .esc = "\e[6~", .key = NCKEY_PGDOWN, },
-    { .esc = "\eOP", .key = NCKEY_F01, },
-    { .esc = "\eOQ", .key = NCKEY_F02, },
-    { .esc = "\eOR", .key = NCKEY_F03, },
-    { .esc = "\eOS", .key = NCKEY_F04, },
-    { .esc = "\e[15~", .key = NCKEY_F05, },
-    { .esc = "\e[17~", .key = NCKEY_F06, },
-    { .esc = "\e[18~", .key = NCKEY_F07, },
-    { .esc = "\e[19~", .key = NCKEY_F08, },
-    { .esc = "\e[20~", .key = NCKEY_F09, },
-    { .esc = "\e[21~", .key = NCKEY_F10, },
-    { .esc = "\e[23~", .key = NCKEY_F11, },
-    { .esc = "\e[24~", .key = NCKEY_F12, },
+    { .esc = "\x1b[A", .key = NCKEY_UP, },
+    { .esc = "\x1b[B", .key = NCKEY_DOWN, },
+    { .esc = "\x1b[C", .key = NCKEY_RIGHT, },
+    { .esc = "\x1b[D", .key = NCKEY_LEFT, },
+    { .esc = "\x1b[1;5A", .key = NCKEY_UP, .ctrl = 1, },
+    { .esc = "\x1b[1;5B", .key = NCKEY_DOWN, .ctrl = 1, },
+    { .esc = "\x1b[1;5C", .key = NCKEY_RIGHT, .ctrl = 1, },
+    { .esc = "\x1b[1;5D", .key = NCKEY_LEFT, .ctrl = 1, },
+    { .esc = "\x1b[H", .key = NCKEY_HOME, },
+    { .esc = "\x1b[F", .key = NCKEY_END, },
+    { .esc = "\x1b[2~", .key = NCKEY_INS, },
+    { .esc = "\x1b[3~", .key = NCKEY_DEL, },
+    { .esc = "\x1b[5~", .key = NCKEY_PGUP, },
+    { .esc = "\x1b[6~", .key = NCKEY_PGDOWN, },
+    { .esc = "\x1bOP", .key = NCKEY_F01, },
+    { .esc = "\x1bOQ", .key = NCKEY_F02, },
+    { .esc = "\x1bOR", .key = NCKEY_F03, },
+    { .esc = "\x1bOS", .key = NCKEY_F04, },
+    { .esc = "\x1b[15~", .key = NCKEY_F05, },
+    { .esc = "\x1b[17~", .key = NCKEY_F06, },
+    { .esc = "\x1b[18~", .key = NCKEY_F07, },
+    { .esc = "\x1b[19~", .key = NCKEY_F08, },
+    { .esc = "\x1b[20~", .key = NCKEY_F09, },
+    { .esc = "\x1b[21~", .key = NCKEY_F10, },
+    { .esc = "\x1b[23~", .key = NCKEY_F11, },
+    { .esc = "\x1b[24~", .key = NCKEY_F12, },
     { .esc = NULL, .key = NCKEY_INVALID, },
   }, *k;
   for(k = keys ; k->esc ; ++k){
@@ -871,17 +961,17 @@ prep_special_keys(ncinputlayer* nc){
   for(k = keys ; k->tinfo ; ++k){
     char* seq = tigetstr(k->tinfo);
     if(seq == NULL || seq == (char*)-1){
-//fprintf(stderr, "no support for terminfo's %s\n", k->tinfo);
+      loginfo("no terminfo declaration for %s\n", k->tinfo);
       continue;
     }
-    if(seq[0] != NCKEY_ESC){
-      loginfo("%s is not an escape sequence\n", k->tinfo);
+    if(seq[0] != NCKEY_ESC || strlen(seq) < 2){ // assume ESC prefix + content
+      logwarn("invalid escape: %s (0x%x)\n", k->tinfo, k->key);
       continue;
     }
-    logdebug("support for terminfo's %s: %s\n", k->tinfo, seq);
     if(ncinputlayer_add_input_escape(nc, seq, k->key, k->shift, k->ctrl, k->alt)){
       return -1;
     }
+    logdebug("support for terminfo's %s: %s\n", k->tinfo, seq);
   }
   if(ncinputlayer_add_input_escape(nc, CSIPREFIX, NCKEY_CSI, 0, 0, 0)){
     return -1;
@@ -935,20 +1025,11 @@ typedef enum {
   STATE_SDA,  // secondary DA (CSI > Pp ; Pv ; Pc c)
   STATE_SDA_VER,  // secondary DA, got semi, reading to next semi
   STATE_SDA_DRAIN, // drain secondary DA to 'c'
-  STATE_DA,   // primary DA   (CSI ? ... c) OR XTSMGRAPHICS OR DECRPM
-  STATE_DA_1, // got '1', XTSMGRAPHICS color registers or primary DA
-  STATE_DA_1_SEMI, // got '1;'
-  STATE_DA_1_0, // got '1;0', XTSMGRAPHICS color registers or VT101
-  STATE_DA_1_SEMI_2, // got '1;2', XTSMGRAPHICS color reg fail or VT100
-  STATE_DA_6, // got '6', could be VT102 or VT220/VT320/VT420
+  STATE_DA,   // primary DA   (CSI ? ... c) OR XTSMGRAPHICS OR DECRPM or kittykbd
   STATE_DA_DRAIN, // drain out the primary DA to an alpha
-  STATE_SIXEL,// XTSMGRAPHICS about Sixel geometry (got '2')
-  STATE_SIXEL_SEMI1,   // got first semicolon in sixel geometry, want Ps
-  STATE_SIXEL_SUCCESS, // got Ps == 0, want second semicolon
-  STATE_SIXEL_WIDTH,   // reading maximum sixel width until ';'
-  STATE_SIXEL_HEIGHT,  // reading maximum sixel height until 'S'
-  STATE_SIXEL_CREGS,   // reading max color registers until 'S'
-  STATE_XTSMGRAPHICS_DRAIN, // drain out XTSMGRAPHICS to 'S'
+  STATE_DA_SEMI,    // got first semicolon following numeric
+  STATE_DA_SEMI2,   // got second semicolon following numeric ; numeric
+  STATE_DA_SEMI3,   // got third semicolon following numeric ; numeric ; numeric
   STATE_APPSYNC_REPORT, // got DECRPT ?2026
   STATE_APPSYNC_REPORT_DRAIN, // drain out decrpt to 'y'
   // cursor location report: CSI row ; col R
@@ -982,6 +1063,8 @@ typedef struct query_state {
   int dimy;              // screen height in cells
   int cursor_y, cursor_x;// cursor location
   int cursor_or_pixel;   // holding cell until we determine which state
+  int three;             // third param (XTSMGRAPHICS)
+  int four;              // fourth param (XTSMGRAPHICS)
 
   bool xtgettcap_good;   // high when we've received DCS 1
   bool appsync;          // application-synchronized updates advertised
@@ -1247,7 +1330,7 @@ pump_control_read(query_state* inits, unsigned char c){
       break;
     case STATE_CSI: // terminated by 0x40--0x7E ('@'--'~')
       if(c == '?'){
-        inits->state = STATE_DA; // could also be DECRPM/XTSMGRAPHICS
+        inits->state = STATE_DA; // could also be DECRPM/XTSMGRAPHICS/kittykbd
       }else if(c == '>'){
         // SDA yields up Alacritty's crate version, but it doesn't unambiguously
         // identify Alacritty. If we've got any other version information, skip
@@ -1484,148 +1567,105 @@ pump_control_read(query_state* inits, unsigned char c){
     //     CSI ? 6 2 ; Ps c  ("VT220")
     //     CSI ? 6 3 ; Ps c  ("VT320")
     //     CSI ? 6 4 ; Ps c  ("VT420")
+    // KITTYKBD: CSI ? flags u
     case STATE_DA: // return success on end of DA
 //fprintf(stderr, "DA: %c\n", c);
-      if(c == '1'){
-        inits->state = STATE_DA_1;
-      }else if(c == '2'){
-        if(ruts_numeric(&inits->numeric, c)){ // stash for DECRPT
+      // FIXME several of these numbers could be DECRPM/XTSM/kittykbd. probably
+      // just want to read number, *then* make transition on non-number.
+      if(isdigit(c)){
+        if(ruts_numeric(&inits->numeric, c)){ // stash for DECRPM/XTSM/kittykbd
           return -1;
         }
-        inits->state = STATE_SIXEL;
-      }else if(c == '4' || c == '7'){ // VT132, VT131
-        inits->state = STATE_DA_DRAIN;
-      }else if(c == '6'){
-        inits->state = STATE_DA_6;
-      }else if(c >= 0x40 && c <= 0x7E){
-        inits->state = STATE_NULL;
-      }
-      break;
-    case STATE_DA_1:
-//fprintf(stderr, "DA1: %c\n", c);
-      if(c == 'c'){
-        inits->state = STATE_NULL;
-        return 1;
-      }else if(c >= 0x40 && c <= 0x7E){
+      }else if(c == 'u'){ // kitty keyboard
+        inits->tcache->kittykbd = inits->numeric;
+        loginfo("keyboard protocol 0x%x\n", inits->tcache->kittykbd);
         inits->state = STATE_NULL;
       }else if(c == ';'){
-        inits->state = STATE_DA_1_SEMI;
-      } // FIXME error?
-      break;
-    case STATE_DA_1_SEMI:
-//fprintf(stderr, "DA1SEMI: %c\n", c);
-      if(c == '2'){ // VT100 with Advanced Video Option *or* setcregs failure
-        inits->state = STATE_DA_1_SEMI_2;
-      }else if(c == '0'){
-        inits->state = STATE_DA_1_0;
+        inits->cursor_or_pixel = inits->numeric;
+        inits->numeric = 0;
+        inits->state = STATE_DA_SEMI;
       }else if(c >= 0x40 && c <= 0x7E){
         inits->state = STATE_NULL;
-      } // FIXME error?
+        if(c == 'c'){
+          return 1;
+        }
+      }
       break;
-    case STATE_DA_1_SEMI_2:
-      if(c == 'c'){
+    case STATE_DA_SEMI:
+      if(c == ';'){
+        inits->three = inits->numeric;
+        inits->numeric = 0;
+        inits->state = STATE_DA_SEMI2;
+      }else if(isdigit(c)){
+        if(ruts_numeric(&inits->numeric, c)){
+          return -1;
+        }
+      }else if(c == '$'){
+        if(inits->cursor_or_pixel == 2026){
+          inits->state = STATE_APPSYNC_REPORT;
+          loginfo("terminal reported SUM support\n");
+        }else{
+          inits->state = STATE_APPSYNC_REPORT_DRAIN;
+        }
+      }else if(c >= 0x40 && c <= 0x7E){
         inits->state = STATE_NULL;
-        return 1;
+        if(c == 'c'){
+          return 1;
+        }
+      }
+      break;
+    case STATE_DA_SEMI2:
+      if(c == ';'){
+        inits->four = inits->numeric;
+        inits->numeric = 0;
+        inits->state = STATE_DA_SEMI3;
+      }else if(isdigit(c)){
+        if(ruts_numeric(&inits->numeric, c)){
+          return -1;
+        }
       }else if(c == 'S'){
+        if(inits->cursor_or_pixel == 1){
+          inits->tcache->color_registers = inits->numeric;
+          loginfo("sixel color registers: %d\n", inits->tcache->color_registers);
+          inits->numeric = 0;
+        }
         inits->state = STATE_NULL;
       }else if(c >= 0x40 && c <= 0x7E){
         inits->state = STATE_NULL;
+        if(c == 'c'){
+          return 1;
+        }
       }
       break;
-    case STATE_DA_1_0:
-//fprintf(stderr, "DA_!_0: %c\n", c);
-      if(c == 'c'){
-        inits->state = STATE_NULL;
-        return 1;
-      }else if(c >= 0x40 && c <= 0x7E){
-        inits->state = STATE_NULL;
-      }else if(c == ';'){
-        inits->state = STATE_SIXEL_CREGS;
-      } // FIXME error?
-      break;
-    case STATE_SIXEL_CREGS:
-      if(c == 'S'){
-//fprintf(stderr, "CREGS: %d\n", inits->numeric);
-        inits->tcache->color_registers = inits->numeric;
-        inits->state = STATE_NULL;
-      }else if(ruts_numeric(&inits->numeric, c)){
-        return -1;
-      }
-      break;
-    case STATE_DA_6:
-      if(c == 'c'){
-        inits->state = STATE_NULL;
-        return 1;
-      }else if(c >= 0x40 && c <= 0x7E){
-        inits->state = STATE_NULL;
-      }
-      // FIXME
-      break;
     case STATE_DA_DRAIN:
-      if(c == 'c'){
+      if(c >= 0x40 && c <= 0x7E){
         inits->state = STATE_NULL;
-        return 1;
+        if(c == 'c'){
+          return 1;
+        }
+      }
+      break;
+    case STATE_DA_SEMI3:
+      if(c == ';'){
+        inits->numeric = 0;
+        inits->state = STATE_DA_DRAIN;
+      }else if(isdigit(c)){
+        if(ruts_numeric(&inits->numeric, c)){
+          return -1;
+        }
+      }else if(c == 'S'){
+        inits->tcache->sixel_maxx = inits->four;
+        inits->tcache->sixel_maxy = inits->numeric;
+        loginfo("max sixel geometry: %dx%d\n", inits->tcache->sixel_maxy, inits->tcache->sixel_maxx);
       }else if(c >= 0x40 && c <= 0x7E){
         inits->state = STATE_NULL;
-      }
-      break;
-    case STATE_SIXEL:
-      if(c == ';'){
-        if(inits->numeric == 2026){
-          inits->state = STATE_APPSYNC_REPORT;
-        }else{
-          inits->state = STATE_SIXEL_SEMI1;
+        if(c == 'c'){
+          return 1;
         }
-      }else if(ruts_numeric(&inits->numeric, c)){
-        return -1;
-      }else{
-        // FIXME error?
-      }
-      break;
-    case STATE_SIXEL_SEMI1:
-//fprintf(stderr, "SIXLE`l`: %c\n", c);
-      if(c == '0'){
-        inits->state = STATE_SIXEL_SUCCESS;
-      }else if(c == '2'){
-        inits->state = STATE_XTSMGRAPHICS_DRAIN;
-      }else{
-        // FIXME error?
-      }
-      break;
-    case STATE_SIXEL_SUCCESS:
-      if(c == ';'){
-        inits->numeric = 0;
-        inits->state = STATE_SIXEL_WIDTH;
-      }else{
-        // FIXME error?
-      }
-      break;
-    case STATE_SIXEL_WIDTH:
-      if(c == ';'){
-//fprintf(stderr, "HEIGHT: %d\n", inits->numeric);
-        inits->tcache->sixel_maxx = inits->numeric;
-        inits->state = STATE_SIXEL_HEIGHT;
-        inits->numeric = 0;
-      }else if(ruts_numeric(&inits->numeric, c)){
-        return -1;
-      }
-      break;
-    case STATE_SIXEL_HEIGHT:
-      if(c == 'S'){
-//fprintf(stderr, "HEIGHT: %d\n", inits->numeric);
-        inits->tcache->sixel_maxy_pristine = inits->numeric;
-        inits->state = STATE_NULL;
-      }else if(ruts_numeric(&inits->numeric, c)){
-        return -1;
-      }
-      break;
-    case STATE_XTSMGRAPHICS_DRAIN:
-      if(c == 'S'){
-        inits->state = STATE_NULL;
       }
       break;
     case STATE_APPSYNC_REPORT:
-      if(c == '2'){
+      if(inits->numeric == '2'){
         inits->appsync = 1;
         inits->state = STATE_APPSYNC_REPORT_DRAIN;
       }
@@ -1674,6 +1714,22 @@ err:
   return -1;
 }
 
+static int
+prep_all_keys(ncinputlayer* ni){
+  if(prep_windows_special_keys(ni)){
+    return -1;
+  }
+  if(prep_kitty_special_keys(ni)){
+    input_free_esctrie(&ni->inputescapes);
+    return -1;
+  }
+  if(prep_special_keys(ni)){
+    input_free_esctrie(&ni->inputescapes);
+    return -1;
+  }
+  return 0;
+}
+
 int ncinputlayer_init(tinfo* tcache, FILE* infp, queried_terminals_e* detected,
                       unsigned* appsync, int* cursor_y, int* cursor_x,
                       ncsharedstats* stats, unsigned* kittygraphs){
@@ -1686,12 +1742,7 @@ int ncinputlayer_init(tinfo* tcache, FILE* infp, queried_terminals_e* detected,
   nilayer->infd = fileno(infp);
   loginfo("input fd: %d\n", nilayer->infd);
   nilayer->ttyfd = tty_check(nilayer->infd) ? -1 : get_tty_fd(infp);
-  if(*detected == TERMINAL_MSTERMINAL){
-    if(prep_windows_special_keys(nilayer)){
-      return -1;
-    }
-  }
-  if(prep_special_keys(nilayer)){
+  if(prep_all_keys(nilayer)){
     pthread_mutex_destroy(&nilayer->lock);
     return -1;
   }
@@ -1729,14 +1780,18 @@ int ncinputlayer_init(tinfo* tcache, FILE* infp, queried_terminals_e* detected,
     if(cursor_y){
       *cursor_y = inits.cursor_y - 1;
     }
-    if(inits.pixelwidth && inits.pixelheight){
-      tcache->pixy = inits.pixelheight;
-      tcache->pixx = inits.pixelwidth;
-    }
     if(inits.dimy && inits.dimx){
       tcache->default_rows = inits.dimy;
       tcache->default_cols = inits.dimx;
     }
+    if(inits.pixelwidth && inits.pixelheight){
+      tcache->pixy = inits.pixelheight;
+      tcache->pixx = inits.pixelwidth;
+      if(tcache->default_rows && tcache->default_cols){
+        tcache->cellpixx = tcache->pixx / tcache->default_cols;
+        tcache->cellpixy = tcache->pixy / tcache->default_rows;
+      }
+    }
     if(inits.kittygraphics){ // kitty trumps sixel
       loginfo("advertised kitty; disabling sixel\n");
       tcache->color_registers = 0;
@@ -1768,6 +1823,7 @@ scan_for_clrs(ncinputlayer* ni){
       pop_input_keypress(ni);
       pop_input_keypress(ni);
       ncinput nin;
+      // FIXME need real margins here, no?
       if(handle_csi(ni, &nin, 0, 0) == NCKEY_CURSOR_LOCATION_REPORT){
         enqueue_cursor_report(ni, &nin);
       }
@@ -1783,6 +1839,7 @@ scan_for_clrs(ncinputlayer* ni){
 void ncinput_extract_clrs(tinfo* ti){
   ncinputlayer* ni = &ti->input;
   do{
+    // FIXME doesn't this need locking?
     if(ni->inputbuf_occupied){
       scan_for_clrs(ni);
       if(ni->creport_queue){
