@@ -3,7 +3,6 @@
 #include "version.h"
 #include "egcpool.h"
 #include "internal.h"
-#include <zlib.h>
 #include <time.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -18,6 +17,7 @@
 #include <inttypes.h>
 #include <notcurses/direct.h>
 #include "compat/compat.h"
+#include "banner.h"
 
 #define ESC "\x1b"
 
@@ -92,7 +92,15 @@ notcurses_stop_minimal(void* vnc){
   }
   ret |= mouse_disable(&nc->tcache, f);
   ret |= reset_term_attributes(&nc->tcache, f);
+  if(nc->tcache.kittykbd){
+    if(fbuf_emit(f, "\x1b[<u")){
+      ret = -1;
+    }
+  }
   if(nc->tcache.ttyfd >= 0){
+    if(nc->tcache.tpreserved){
+      ret |= tcsetattr(nc->tcache.ttyfd, TCSAFLUSH, nc->tcache.tpreserved);
+    }
     if((esc = get_escape(&nc->tcache, ESCAPE_RMCUP))){
       if(sprite_clear_all(&nc->tcache, f)){
         ret = -1;
@@ -101,7 +109,6 @@ notcurses_stop_minimal(void* vnc){
         ret = -1;
       }
     }
-    ret |= tcsetattr(nc->tcache.ttyfd, TCSAFLUSH, nc->tcache.tpreserved);
   }
   if((esc = get_escape(&nc->tcache, ESCAPE_RMKX)) && fbuf_emit(f, esc)){
     ret = -1;
@@ -110,7 +117,10 @@ notcurses_stop_minimal(void* vnc){
   if(cnorm && fbuf_emit(f, cnorm)){
     ret = -1;
   }
-  return blocking_write(fileno(nc->ttyfp), f->buf, f->used);
+  if(blocking_write(fileno(nc->ttyfp), f->buf, f->used)){
+    ret = -1;
+  }
+  return ret;
 }
 
 // make a heap-allocated wchar_t expansion of the multibyte string at s
@@ -913,86 +923,6 @@ int ncplane_destroy_family(ncplane *ncp){
   return ret;
 }
 
-// only invoked without suppress banners flag. prints various warnings based on
-// the environment / terminal definition. returns the number of lines printed.
-static int
-init_banner_warnings(const notcurses* nc, fbuf* f){
-  term_fg_palindex(nc, f, nc->tcache.caps.colors <= 88 ? 1 : 0xcb);
-  if(!notcurses_canutf8(nc)){
-    fbuf_puts(f, " Warning! Encoding is not UTF-8; output may be degraded.\n");
-  }
-  if(!get_escape(&nc->tcache, ESCAPE_HPA)){
-    fbuf_puts(f, " Warning! No absolute horizontal placement.\n");
-  }
-  return 0;
-}
-
-// unless the suppress_banner flag was set, print some version information and
-// (if applicable) warnings to ttyfp. we are not yet on the alternate screen.
-static int
-init_banner(const notcurses* nc, fbuf* f){
-  if(!nc->suppress_banner){
-    term_fg_palindex(nc, f, 50 % nc->tcache.caps.colors);
-    fbuf_printf(f, "notcurses %s on %s %s\n", notcurses_version(),
-                nc->tcache.termname ? nc->tcache.termname : "?",
-                nc->tcache.termversion ? nc->tcache.termversion : "");
-    term_fg_palindex(nc, f, nc->tcache.caps.colors <= 256 ?
-                     14 % nc->tcache.caps.colors : 0x2080e0);
-    if(nc->tcache.cellpixy && nc->tcache.cellpixx){
-      fbuf_printf(f, "%d rows (%dpx) %d cols (%dpx) %dx%d ",
-                  nc->stdplane->leny, nc->tcache.cellpixy,
-                  nc->stdplane->lenx, nc->tcache.cellpixx,
-                  nc->stdplane->leny * nc->tcache.cellpixy,
-                  nc->stdplane->lenx * nc->tcache.cellpixx);
-    }else{
-      fbuf_printf(f, "%d rows %d cols ",
-                  nc->stdplane->leny, nc->stdplane->lenx);
-    }
-    const char* setaf;
-    if(nc->tcache.caps.rgb && (setaf = get_escape(&nc->tcache, ESCAPE_SETAF))){
-      term_fg_rgb8(&nc->tcache, f, 0xe0, 0x60, 0x60);
-      fbuf_putc(f, 'r');
-      term_fg_rgb8(&nc->tcache, f, 0x60, 0xe0, 0x60);
-      fbuf_putc(f, 'g');
-      term_fg_rgb8(&nc->tcache, f, 0x20, 0x80, 0xff);
-      fbuf_putc(f, 'b');
-      term_fg_palindex(nc, f, nc->tcache.caps.colors <= 256 ?
-                       14 % nc->tcache.caps.colors : 0x2080e0);
-      fbuf_putc(f, '+');
-    }
-    fbuf_printf(f, "%u colors\n%s%s (%s)\nterminfo from %s zlib %s GPM %s\n",
-                nc->tcache.caps.colors,
-#ifdef __clang__
-            "", // name is contained in __VERSION__
-#else
-#ifdef __GNUC__
-            "gcc-",
-#else
-#error "Unknown compiler"
-#endif
-#endif
-            __VERSION__,
-#ifdef __BYTE_ORDER__
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-            "LE",
-#else
-            "BE",
-#endif
-#else
-#error "No __BYTE_ORDER__ definition"
-#endif
-            curses_version(), zlibVersion(), gpm_version());
-    ncvisual_printbanner(f);
-    init_banner_warnings(nc, f);
-    const char* esc;
-    if( (esc = get_escape(&nc->tcache, ESCAPE_SGR0)) ||
-        (esc = get_escape(&nc->tcache, ESCAPE_OP))){
-      fbuf_emit(f, esc);
-    }
-  }
-  return 0;
-}
-
 // it's critical that we're using UTF-8 encoding if at all possible. since the
 // client might not have called setlocale(2) (if they weren't reading the
 // directions...), go ahead and try calling setlocale(LC_ALL, "") and then
@@ -1099,6 +1029,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   }else{
     fprintf(stderr, "Encoding (\"%s\") was neither ANSI_X3.4-1968 nor UTF-8, refusing to start\n Did you call setlocale()?\n",
             encoding ? encoding : "none found");
+    free(ret);
     return NULL;
   }
   ret->flags = opts->flags;
@@ -1165,7 +1096,12 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
                           opts->flags & NCOPTION_NO_ALTERNATE_SCREEN, 0,
                           opts->flags & NCOPTION_NO_FONT_CHANGES,
                           cursory, cursorx, &ret->stats)){
-    goto err;
+    fbuf_free(&ret->rstate.f);
+    pthread_mutex_destroy(&ret->pilelock);
+    pthread_mutex_destroy(&ret->stats.lock);
+    drop_signals(ret);
+    free(ret);
+    return NULL;
   }
   if((opts->flags & NCOPTION_PRESERVE_CURSOR) || !ret->suppress_banner){
     // the u7 led the queries so that we would get a cursor position
@@ -1219,26 +1155,15 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   if(set_fd_nonblocking(ret->tcache.input.infd, 1, &ret->stdio_blocking_save)){
     goto err;
   }
-  // if not connected to an actual terminal, we're not going to try entering
-  // the alternate screen; we're not even going to bother clearing the screen.
-  if(ret->tcache.ttyfd >= 0){
-    if(!(opts->flags & NCOPTION_NO_ALTERNATE_SCREEN)){
-      const char* smcup = get_escape(&ret->tcache, ESCAPE_SMCUP);
-      if(smcup){
-        if(enter_alternate_screen(ret->ttyfp, &ret->tcache, false)){
-          free_plane(ret->stdplane);
-          goto err;
-        }
-      }
-      // perform an explicit clear since the alternate screen was requested
-      // (smcup *might* clear, but who knows? and it might not have been
-      // available in any case).
-      if(clear_and_home(ret, &ret->tcache, &ret->rstate.f)){
-        goto err;
-      }
-      // no need to reestablish a preserved cursor -- that only affects the
-      // standard plane, not the physical cursor that was just disrupted.
+  if(!(opts->flags & NCOPTION_NO_ALTERNATE_SCREEN)){
+    // perform an explicit clear since the alternate screen was requested
+    // (smcup *might* clear, but who knows? and it might not have been
+    // available in any case).
+    if(clear_and_home(ret, &ret->tcache, &ret->rstate.f)){
+      goto err;
     }
+    // no need to reestablish a preserved cursor -- that only affects the
+    // standard plane, not the physical cursor that was just disrupted.
   }
   // the sprite clear ought take place within the alternate screen, if it's
   // being used.
@@ -1258,8 +1183,9 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
 
 err:
   logpanic("Alas, you will not be going to space today.\n");
+  notcurses_stop_minimal(ret);
   fbuf_free(&ret->rstate.f);
-  if(ret->tcache.tpreserved){
+  if(ret->tcache.ttyfd >= 0 && ret->tcache.tpreserved){
     (void)tcsetattr(ret->tcache.ttyfd, TCSAFLUSH, ret->tcache.tpreserved);
     free(ret->tcache.tpreserved);
   }
