@@ -153,7 +153,7 @@ paint_sprixel(ncplane* p, struct crender* rvec, int starty, int startx,
       if(crender->p || crender->s.bgblends){
         // if sprite_wipe_cell() fails, we presumably do not have the
         // ability to wipe, and must reprint the character
-        if(sprite_wipe(nc, p->sprite, y, x)){
+        if(sprite_wipe(nc, p->sprite, y, x) < 0){
 //fprintf(stderr, "damaging due to wipe [%s] %d/%d\n", nccell_extended_gcluster(crender->p, &crender->c), absy, absx);
           crender->s.damaged = 1;
         }
@@ -249,10 +249,9 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
       if(nccell_wide_right_p(targc)){
         continue;
       }
-      const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
 
       if(nccell_fg_alpha(targc) > NCALPHA_OPAQUE){
-        vis = &p->fb[nfbcellidx(p, y, x)];
+        const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
         if(nccell_fg_default_p(vis)){
           vis = &p->basecell;
         }
@@ -284,7 +283,7 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
       // background channel and balpha.
       // Evaluate the background first, in case we have HIGHCONTRAST fg text.
       if(nccell_bg_alpha(targc) > NCALPHA_OPAQUE){
-        vis = &p->fb[nfbcellidx(p, y, x)];
+        const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
         // to be on the blitter stacking path, we need
         //  1) crender->s.blittedquads to be non-zero (we're below semigraphics)
         //  2) cell_blittedquadrants(vis) to be non-zero (we're semigraphics)
@@ -327,7 +326,7 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
       // still use a character we find here, but its color will come entirely
       // from cells underneath us.
       if(!crender->p){
-        vis = &p->fb[nfbcellidx(p, y, x)];
+        const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
         if(vis->gcluster == 0 && !nccell_double_wide_p(vis)){
           vis = &p->basecell;
         }
@@ -805,7 +804,7 @@ emit_bg_palindex(notcurses* nc, fbuf* f, const nccell* srccell){
 // this first phase of sprixel rasterization is responsible for:
 //  1) invalidating all QUIESCENT sprixels if the pile has changed (because
 //      it would have been destroyed when switching away from our pile).
-//      for the same reason, invalidated all MOVE sprixels in this case.
+//      for the same reason, invalidate all MOVE sprixels in this case.
 //  2) damaging all cells under a HIDE sixel, so text phase 1 consumes it
 //      (not necessary for kitty graphics)
 //  3) damaging uncovered cells under a MOVE (not necessary for kitty)
@@ -823,7 +822,7 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
     loginfo("Phase 1 sprixel %u state %d\n", s->id, s->invalidated);
     if(s->invalidated == SPRIXEL_QUIESCENT){
       if(p != nc->last_pile){
-        s->invalidated = SPRIXEL_INVALIDATED;
+        s->invalidated = SPRIXEL_UNSEEN;
       }
     }else if(s->invalidated == SPRIXEL_HIDE){
 //fprintf(stderr, "OUGHT HIDE %d [%dx%d] %p\n", s->id, s->dimy, s->dimx, s);
@@ -835,18 +834,21 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
           s->next->prev = s->prev;
         }
         sprixel_free(s);
-      }else{
-        parent = &s->next;
+        // need to avoid the rest of the iteration, as s is dead
+        continue; // don't account as an elision
       }
-      continue; // don't account as an elision
     }
-    if(s->invalidated == SPRIXEL_MOVED || s->invalidated == SPRIXEL_INVALIDATED || s->invalidated == SPRIXEL_UNSEEN){
+    if(s->invalidated == SPRIXEL_INVALIDATED && nc->tcache.pixel_refresh){
+      nc->tcache.pixel_refresh(p, s);
+    }else if(s->invalidated == SPRIXEL_MOVED ||
+             s->invalidated == SPRIXEL_UNSEEN ||
+             s->invalidated == SPRIXEL_INVALIDATED){
       int y, x;
       ncplane_abs_yx(s->n, &y, &x);
 //fprintf(stderr, "1 MOVING BITMAP %d STATE %d AT %d/%d for %p\n", s->id, s->invalidated, y + nc->margin_t, x + nc->margin_l, s->n);
       if(s->invalidated == SPRIXEL_MOVED){
         if(p != nc->last_pile){
-          s->invalidated = SPRIXEL_INVALIDATED;
+          s->invalidated = SPRIXEL_UNSEEN;
         }else{
           if(s->n->absx == s->movedfromx){
             if(s->movedfromy - s->n->absy == scrolls){
@@ -856,21 +858,18 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
             }
           }
         }
-        // otherwise it's a new pile, so we couldn't have been on-screen
       }
-      int r = sprite_redraw(&nc->tcache, p, s, f, y + nc->margin_t, x + nc->margin_l);
+      // otherwise it's a new pile, so we couldn't have been on-screen
+      int r = sprite_redraw(nc, p, s, f, y + nc->margin_t, x + nc->margin_l);
       if(r < 0){
         return -1;
       }
       bytesemitted += r;
-      // FIXME might not need this if it was only an upload
-      nc->rstate.hardcursorpos = true;
-      parent = &s->next;
       ++nc->stats.s.sprixelemissions;
     }else{
       ++nc->stats.s.sprixelelisions;
-      parent = &s->next;
     }
+    parent = &s->next;
 //fprintf(stderr, "SPRIXEL STATE: %d\n", s->invalidated);
   }
   return bytesemitted;
@@ -945,7 +944,6 @@ rasterize_scrolls(const ncpile* p, fbuf* f){
       if(fbuf_emit(f, tiparm(indn, scrolls)) < 0){
         return -1;
       }
-      scrolls = 0;
       return 0;
     }
   }
@@ -1075,7 +1073,7 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
           ++x;
         }
       }else if(phase != 0 || !rvec[damageidx].s.p_beats_sprixel){
-//fprintf(stderr, "phase %u damaged at %d/%d\n", phase, innery, innerx);
+//fprintf(stderr, "phase %u damaged at %d/%d %d\n", phase, innery, innerx, x);
         // in the first text phase, we draw only those glyphs where the glyph
         // was not above a sprixel (and the cell is damaged). in the second
         // phase, we draw everything that remains damaged.
@@ -1164,10 +1162,11 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
         }
         rvec[damageidx].s.damaged = 0;
         rvec[damageidx].s.p_beats_sprixel = 0;
-        ++nc->rstate.x;
-        if(srccell->width >= 2){
+        nc->rstate.x += srccell->width;
+        if(srccell->width){ // check only necessary when undamaged; be safe
           x += srccell->width - 1;
-          nc->rstate.x += srccell->width - 1;
+        }else{
+          ++nc->rstate.x;
         }
       }
 //fprintf(stderr, "damageidx: %ld\n", damageidx);
