@@ -55,6 +55,7 @@ setup_sixel_bitmaps(tinfo* ti, int fd, bool invert80){
   ti->pixel_scrub = sixel_scrub;
   ti->pixel_remove = NULL;
   ti->pixel_draw = sixel_draw;
+  ti->pixel_refresh = sixel_refresh;
   ti->pixel_draw_late = NULL;
   ti->pixel_commit = NULL;
   ti->pixel_move = NULL;
@@ -78,6 +79,7 @@ setup_kitty_bitmaps(tinfo* ti, int fd, kitty_graphics_e level){
   ti->pixel_remove = kitty_remove;
   ti->pixel_draw = kitty_draw;
   ti->pixel_draw_late = NULL;
+  ti->pixel_refresh = NULL;
   ti->pixel_commit = kitty_commit;
   ti->pixel_move = kitty_move;
   ti->pixel_scroll = NULL;
@@ -113,6 +115,7 @@ setup_fbcon_bitmaps(tinfo* ti, int fd){
   ti->pixel_draw = NULL;
   ti->pixel_draw_late = fbcon_draw;
   ti->pixel_commit = NULL;
+  ti->pixel_refresh = NULL;
   ti->pixel_move = NULL;
   ti->pixel_scroll = fbcon_scroll;
   ti->pixel_shutdown = NULL;
@@ -317,10 +320,20 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // which can be identified directly, sans queries.
 #define KITTYQUERY "\x1b_Gi=1,a=q;\x1b\\"
 
+// request kitty keyboard protocol through level 31, and push current.
+// see https://sw.kovidgoyal.net/kitty/keyboard-protocol/#progressive-enhancement
+#define KBDSUPPORT "\x1b[>u\x1b[=1u"
+
+// the kitty keyboard protocol allows unambiguous, complete identification of
+// input events. this queries for the level of support.
+#define KBDQUERY "\x1b[?u"
+
 // these queries (terminated with a Primary Device Attributes, to which
 // all known terminals reply) hopefully can uniquely and unquestionably
 // identify the terminal to which we are talking.
 #define IDQUERIES KITTYQUERY \
+                  KBDSUPPORT \
+                  KBDQUERY \
                   TRIDEVATTR \
                   XTVERSION \
                   XTGETTCAPTN \
@@ -362,18 +375,31 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
                    GEOMCELL \
                    PRIDEVATTR
 
+// enter the alternate screen (smcup). we could technically get this from
+// terminfo, but everyone who supports it supports it the same way, and we
+// need to send it before our other directives if we're going to use it.
+#define SMCUP "\x1b[?1049h"
+
 // we send an XTSMGRAPHICS to set up 256 color registers (the most we can
 // currently take advantage of; we need at least 64 to use sixel at all).
 // maybe that works, maybe it doesn't. then query both color registers
 // and geometry. send XTGETTCAP for terminal name. if 'minimal' is set, don't
 // send any identification queries (we've already identified the terminal).
 static int
-send_initial_queries(int fd, bool minimal){
+send_initial_queries(int fd, bool minimal, bool noaltscreen){
   const char *queries;
-  if(minimal){
-    queries = DSRCPR DIRECTIVES;
+  if(noaltscreen){
+    if(minimal){
+      queries = DSRCPR DIRECTIVES;
+    }else{
+      queries = DSRCPR IDQUERIES DIRECTIVES;
+    }
   }else{
-    queries = DSRCPR IDQUERIES DIRECTIVES;
+    if(minimal){
+      queries = SMCUP DSRCPR DIRECTIVES;
+    }else{
+      queries = SMCUP DSRCPR IDQUERIES DIRECTIVES;
+    }
   }
   size_t len = strlen(queries);
   loginfo("sending %lluB queries\n", (unsigned long long)len);
@@ -741,11 +767,11 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
       free(ti->tpreserved);
       return -1;
     }
+// FIXME need to enter alternate screen here
     // if we already know our terminal (e.g. on the linux console), there's no
     // need to send the identification queries. the controls are sufficient.
     bool minimal = (ti->qterm != TERMINAL_UNKNOWN);
-    if(send_initial_queries(ti->ttyfd, minimal)){
-      free(ti->tpreserved);
+    if(send_initial_queries(ti->ttyfd, minimal, noaltscreen)){
       goto err;
     }
   }
@@ -755,7 +781,6 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   int termerr;
   if(setupterm(termtype, ti->ttyfd, &termerr)){
     logpanic("Terminfo error %d for %s (see terminfo(3ncurses))\n", termerr, termtype);
-    free(ti->tpreserved);
     goto err;
   }
   tname = termname(); // longname() is also available
@@ -799,6 +824,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
     { ESCAPE_U7, "u7", },
     { ESCAPE_SMKX, "smkx", },
     { ESCAPE_SMXX, "smxx", },
+    { ESCAPE_EL, "el", },
     { ESCAPE_RMXX, "rmxx", },
     { ESCAPE_SMUL, "smul", },
     { ESCAPE_RMUL, "rmul", },
@@ -849,6 +875,15 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
        init_terminfo_esc(ti, "rmcup", ESCAPE_RMCUP, &tablelen, &tableused)){
       goto err;
     }
+    const char* smcup = get_escape(ti, ESCAPE_SMCUP);
+    if(smcup){
+      ti->in_alt_screen = 1;
+      // if we're not using the standard smcup, our initial hardcoded use of it
+      // presumably had no effect; warn the user.
+      if(strcmp(smcup, SMCUP)){
+        logwarn("warning: non-standard smcup!\n");
+      }
+    }
   }else{
     ti->escindices[ESCAPE_SMCUP] = 0;
     ti->escindices[ESCAPE_RMCUP] = 0;
@@ -921,13 +956,17 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   return 0;
 
 err:
+  // FIXME need to leave alternate screen if we entered it
   if(ti->tpreserved){
-    tcsetattr(ti->ttyfd, TCSANOW, ti->tpreserved);
+    (void)tcsetattr(ti->ttyfd, TCSANOW, ti->tpreserved);
+    free(ti->tpreserved);
+    ti->tpreserved = NULL;
   }
-  free(ti->tpreserved);
   free(ti->esctable);
   free(ti->termversion);
   del_curterm(cur_term);
+  close(ti->ttyfd);
+  ti->ttyfd = -1;
   return -1;
 }
 
@@ -953,29 +992,40 @@ char* termdesc_longterm(const tinfo* ti){
 // on the response, easy peasy.
 // FIXME still, we ought reuse buffer, and pass on any excess reads...
 static int
-locate_cursor_simple(int fd, const char* u7, int* cursor_y, int* cursor_x){
+locate_cursor_simple(tinfo* ti, const char* u7, int* cursor_y, int* cursor_x){
+  if(ti->qterm == TERMINAL_MSTERMINAL){
+    return locate_cursor(ti, cursor_y, cursor_x);
+  }
   char* buf = malloc(BUFSIZ);
   if(buf == NULL){
     return -1;
   }
-  if(tty_emit(u7, fd)){
+  loginfo("sending cursor report request\n");
+  if(tty_emit(u7, ti->ttyfd)){
     free(buf);
     return -1;
   }
   ssize_t r;
-  // FIXME rigourize for multiple reads
-  if((r = read(fd, buf, BUFSIZ - 1)) > 0){
-    buf[r] = '\0';
-    if(sscanf(buf, "\e[%d;%dR", cursor_y, cursor_x) != 2){
-      loginfo("Not a cursor location report: %s\n", buf);
-      free(buf);
-      return -1;
+  do{
+    if((r = read(ti->input.infd, buf, BUFSIZ - 1)) > 0){
+      buf[r] = '\0';
+      if(sscanf(buf, "\e[%d;%dR", cursor_y, cursor_x) != 2){
+        loginfo("not a cursor location report: %s\n", buf);
+        free(buf);
+        return -1;
+      }
+      --*cursor_y;
+      --*cursor_x;
+      break;
     }
-    --*cursor_y;
-    --*cursor_x;
+  }while(errno == EAGAIN || errno == EWOULDBLOCK || errno == EBUSY || errno == EINTR);
+  if(r < 0){
+    logerror("error reading cursor location from %d (%s)\n", ti->input.infd, strerror(errno));
+    free(buf);
+    return -1;
   }
   free(buf);
-  loginfo("Located cursor with %d: %d/%d\n", fd, *cursor_y, *cursor_x);
+  loginfo("located cursor with %d: %d/%d\n", ti->ttyfd, *cursor_y, *cursor_x);
   return 0;
 }
 
@@ -983,65 +1033,67 @@ locate_cursor_simple(int fd, const char* u7, int* cursor_y, int* cursor_x){
 // is valid, we can just camp there. otherwise, we need dance with potential
 // user input looking at infd.
 int locate_cursor(tinfo* ti, int* cursor_y, int* cursor_x){
-  if(ti->qterm != TERMINAL_MSTERMINAL){
-    const char* u7 = get_escape(ti, ESCAPE_U7);
-    if(u7 == NULL){
-      logwarn("No support in terminfo\n");
-      return -1;
-    }
-    if(ti->ttyfd >= 0){
-      return locate_cursor_simple(ti->ttyfd, u7, cursor_y, cursor_x);
-    }
-    int fd = ti->input.infd;
-    if(fd < 0){
-      logwarn("No valid path for cursor report\n");
-      return -1;
-    }
-    bool emitted_u7 = false; // only want to send one max
-    cursorreport* clr;
-    loginfo("Acquiring input lock\n");
-    pthread_mutex_lock(&ti->input.lock);
-    while((clr = ti->input.creport_queue) == NULL){
-      logdebug("No report yet\n");
-      if(!emitted_u7){
-        logdebug("Emitting u7\n");
-        // FIXME i'd rather not do this while holding the lock =[
-        if(tty_emit(u7, fd)){
-          pthread_mutex_unlock(&ti->input.lock);
-          return -1;
-        }
-        emitted_u7 = true;
-      }
-      // this can block. we must enter holding the input lock, and it will
-      // return to us holding the input lock.
-      ncinput_extract_clrs(ti);
-      if( (clr = ti->input.creport_queue) ){
-        break;
-      }
-      pthread_cond_wait(&ti->input.creport_cond, &ti->input.lock);
-    }
-    ti->input.creport_queue = clr->next;
-    pthread_mutex_unlock(&ti->input.lock);
-    loginfo("Got a report from %d %d/%d\n", fd, clr->y, clr->x);
-    *cursor_y = clr->y;
-    *cursor_x = clr->x;
-    if(ti->inverted_cursor){
-      int tmp = *cursor_y;
-      *cursor_y = *cursor_x;
-      *cursor_x = tmp;
-    }
-    free(clr);
-    return 0;
-  }
 #ifdef __MINGW64__
-  CONSOLE_SCREEN_BUFFER_INFO conbuf;
-  if(!GetConsoleScreenBufferInfo(ti->outhandle, &conbuf)){
-    logerror("couldn't get cursor info\n");
+  if(ti->qterm == TERMINAL_MSTERMINAL){
+    if(ti->outhandle){
+      CONSOLE_SCREEN_BUFFER_INFO conbuf;
+      if(!GetConsoleScreenBufferInfo(ti->outhandle, &conbuf)){
+        logerror("couldn't get cursor info\n");
+        return -1;
+      }
+      *cursor_y = conbuf.dwCursorPosition.Y;
+      *cursor_x = conbuf.dwCursorPosition.X;
+      loginfo("got a report from y=%d x=%d\n", *cursor_y, *cursor_x);
+      return 0;
+    }
+  }
+#endif
+  const char* u7 = get_escape(ti, ESCAPE_U7);
+  if(u7 == NULL){
+    logwarn("No support in terminfo\n");
     return -1;
   }
-  *cursor_y = conbuf.dwCursorPosition.Y;
-  *cursor_x = conbuf.dwCursorPosition.X;
-  loginfo("got a report from y=%d x=%d\n", *cursor_y, *cursor_x);
-#endif
+  if(ti->ttyfd >= 0){
+    return locate_cursor_simple(ti, u7, cursor_y, cursor_x);
+  }
+  int fd = ti->input.infd;
+  if(fd < 0){
+    logwarn("No valid path for cursor report\n");
+    return -1;
+  }
+  bool emitted_u7 = false; // only want to send one max
+  cursorreport* clr;
+  loginfo("Acquiring input lock\n");
+  pthread_mutex_lock(&ti->input.lock);
+  while((clr = ti->input.creport_queue) == NULL){
+    logdebug("No report yet\n");
+    if(!emitted_u7){
+      logdebug("Emitting u7\n");
+      // FIXME i'd rather not do this while holding the lock =[
+      if(tty_emit(u7, fd)){
+        pthread_mutex_unlock(&ti->input.lock);
+        return -1;
+      }
+      emitted_u7 = true;
+    }
+    // this can block. we must enter holding the input lock, and it will
+    // return to us holding the input lock.
+    ncinput_extract_clrs(ti);
+    if( (clr = ti->input.creport_queue) ){
+      break;
+    }
+    pthread_cond_wait(&ti->input.creport_cond, &ti->input.lock);
+  }
+  ti->input.creport_queue = clr->next;
+  pthread_mutex_unlock(&ti->input.lock);
+  loginfo("Got a report from %d %d/%d\n", fd, clr->y, clr->x);
+  *cursor_y = clr->y;
+  *cursor_x = clr->x;
+  if(ti->inverted_cursor){
+    int tmp = *cursor_y;
+    *cursor_y = *cursor_x;
+    *cursor_x = tmp;
+  }
+  free(clr);
   return 0;
 }
