@@ -54,6 +54,7 @@ typedef enum {
   ESCAPE_CUD,     // "cud" move n cells down
   ESCAPE_SMKX,    // "smkx" keypad_xmit (keypad transmit mode)
   ESCAPE_RMKX,    // "rmkx" keypad_local
+  ESCAPE_EL,      // "el" clear to end of line, inclusive
   ESCAPE_SMCUP,   // "smcup" enter alternate screen
   ESCAPE_RMCUP,   // "rmcup" leave alternate screen
   ESCAPE_SMXX,    // "smxx" start struckout
@@ -105,6 +106,7 @@ typedef struct ncinputlayer {
   int ttyfd; // file descriptor for connected tty
   int infd;  // file descriptor for processing input, from stdin
   unsigned char inputbuf[BUFSIZ];
+  unsigned char csibuf[BUFSIZ];   // running buffer while parsing CSIs
   // we keep a wee ringbuffer of input queued up for delivery. if
   // inputbuf_occupied == sizeof(inputbuf), there is no room. otherwise, data
   // can be read to inputbuf_write_at until we fill up. the first datum
@@ -155,6 +157,10 @@ typedef struct tinfo {
   int (*pixel_wipe)(struct sprixel* s, int y, int x);
   // perform the inverse of pixel_wipe, restoring an annihilated sprixcell.
   int (*pixel_rebuild)(struct sprixel* s, int y, int x, uint8_t* auxvec);
+  // called in phase 1 when INVALIDATED; this damages cells that have been
+  // redrawn in a sixel (when old was not transparent, and new is not opaque).
+  // it leaves the sprixel in INVALIDATED so that it's drawn in phase 2.
+  void (*pixel_refresh)(const struct ncpile* p, struct sprixel* s);
   int (*pixel_remove)(int id, fbuf* f); // kitty only, issue actual delete command
   int (*pixel_init)(const struct tinfo*, int fd); // called when support is detected
   int (*pixel_draw)(const struct tinfo*, const struct ncpile* p,
@@ -198,6 +204,8 @@ typedef struct tinfo {
   int default_rows;          // LINES environment var / lines terminfo / 24
   int default_cols;          // COLUMNS environment var / cols terminfo / 80
 
+  unsigned kittykbd;         // kitty keyboard support level
+
   int gpmfd;                 // connection to GPM daemon
   pthread_t gpmthread;       // thread handle for GPM watcher
 #ifdef __linux__
